/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   Please note: The implementation of this module is partially based on
*   examples that are provided within the GD32 Demo Suite. In case you want
*   to adapt this module to your custom specific hardware, please adapt the
*   hardware initialization code according your needs.
*   This template is responsible to configurate the entire system (CPU clock,
*   memory, qspi, etc).
*
*******************************************************************************/

#include "ewconfig.h"
#include "gd32h7xx.h"
#include "exmc_sdram.h"
#include "ew_bsp_system.h"

static void mpu_config( void )
{
  mpu_region_init_struct mpu_init_struct;
  mpu_region_struct_para_init(&mpu_init_struct);

  /* disable the MPU */
  ARM_MPU_Disable();
  ARM_MPU_SetRegion(0, 0);

  /* configure the MPU attributes for SRAM */
  mpu_init_struct.region_base_address = 0x24000000;
  mpu_init_struct.region_size         = MPU_REGION_SIZE_1MB;
  mpu_init_struct.access_permission   = MPU_AP_FULL_ACCESS;
  mpu_init_struct.access_bufferable   = MPU_ACCESS_NON_BUFFERABLE;
  mpu_init_struct.access_cacheable    = MPU_ACCESS_CACHEABLE;
  mpu_init_struct.access_shareable    = MPU_ACCESS_NON_SHAREABLE;
  mpu_init_struct.region_number       = MPU_REGION_NUMBER0;
  mpu_init_struct.subregion_disable   = MPU_SUBREGION_ENABLE;
  mpu_init_struct.instruction_exec    = MPU_INSTRUCTION_EXEC_PERMIT;
  mpu_init_struct.tex_type            = MPU_TEX_TYPE0;
  mpu_region_config(&mpu_init_struct);
  mpu_region_enable();

  /* configure the MPU attributes for SDRAM to SO */
  mpu_init_struct.region_base_address = 0xC0000000;
  mpu_init_struct.region_size         = MPU_REGION_SIZE_256MB;
  mpu_init_struct.access_permission   = MPU_AP_NO_ACCESS;
  mpu_init_struct.access_bufferable   = MPU_ACCESS_NON_BUFFERABLE;
  mpu_init_struct.access_cacheable    = MPU_ACCESS_NON_CACHEABLE;
  mpu_init_struct.access_shareable    = MPU_ACCESS_NON_SHAREABLE;
  mpu_init_struct.region_number       = MPU_REGION_NUMBER1;
  mpu_init_struct.subregion_disable   = 0x00;
  mpu_init_struct.instruction_exec    = MPU_INSTRUCTION_EXEC_NOT_PERMIT;
  mpu_init_struct.tex_type            = MPU_TEX_TYPE0;
  mpu_region_config(&mpu_init_struct);
  mpu_region_enable();

  /* configure the MPU attributes for SDRAM to WT */
  mpu_init_struct.region_base_address = 0xC0000000;
  mpu_init_struct.region_size         = MPU_REGION_SIZE_32MB;
  mpu_init_struct.access_permission   = MPU_AP_FULL_ACCESS;
  mpu_init_struct.access_bufferable   = MPU_ACCESS_NON_BUFFERABLE;
  mpu_init_struct.access_cacheable    = MPU_ACCESS_CACHEABLE;
  mpu_init_struct.access_shareable    = MPU_ACCESS_NON_SHAREABLE;
  mpu_init_struct.region_number       = MPU_REGION_NUMBER2;
  mpu_init_struct.subregion_disable   = 0x00;
  mpu_init_struct.instruction_exec    = MPU_INSTRUCTION_EXEC_NOT_PERMIT;
  mpu_init_struct.tex_type            = MPU_TEX_TYPE0;
  mpu_region_config(&mpu_init_struct);
  mpu_region_enable();

  /* enable the MPU */
  ARM_MPU_Enable(MPU_MODE_PRIV_DEFAULT);
}


static void SystemClock_Config( void )
{
  /* setup systick timer for 1000Hz interrupts */
  if(SysTick_Config(SystemCoreClock / 1000U)) {
      /* capture error */
      while(1) {
      }
  }
  /* configure the systick handler priority */
  NVIC_SetPriority(SysTick_IRQn, 0x00U);
}


/*******************************************************************************
* FUNCTION:
*   EwBspSystemInit
*
* DESCRIPTION:
*   The function EwBspSystemInit initializes the system components.
*   (CPU clock, memory, qspi, ...)
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspSystemInit( void )
{
  mpu_config();

  /* enable I-Cache */
  SCB_EnableICache();

  /* enable D-Cache */
  SCB_EnableDCache();

  SystemClock_Config();

  /* init SDRAM */
  exmc_synchronous_dynamic_ram_init(EXMC_SDRAM_DEVICE0);
}


/*******************************************************************************
* FUNCTION:
*   EwBspSystemDone
*
* DESCRIPTION:
*   The function EwBspSystemDone terminates the system components.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspSystemDone( void )
{
}

/* msy */
